@@ -1,16 +1,17 @@
 package Test::Spelling;
-use 5.006;
+
 use strict;
 use warnings;
 
-use base 'Exporter';
+use Exporter qw(import);
 use Pod::Spell;
 use Test::Builder;
+use Text::Wrap;
 use File::Spec;
 use IPC::Run3;
 use Symbol 'gensym';
 
-our $VERSION = '0.18';
+our $VERSION = '0.23';
 
 our @EXPORT = qw(
     pod_file_spelling_ok
@@ -28,6 +29,7 @@ my $TEST = Test::Builder->new;
 my $SPELLCHECKER;
 my $FILE_FILTER = sub { 1 };
 my $POD_PARSER;
+our %ALL_WORDS;
 
 sub spellchecker_candidates {
     # if they've specified a spellchecker, use only that one
@@ -35,7 +37,7 @@ sub spellchecker_candidates {
 
     return (
         'spell', # for back-compat, this is the top candidate ...
-        'aspell list -l en', # ... but this should become first soon
+        'aspell list -l en -p /dev/null', # ... but this should become first soon
         'ispell -l',
         'hunspell -l',
     );
@@ -99,9 +101,10 @@ sub invalid_words_in {
 
     my $document = '';
     open my $handle, '>', \$document;
+    open my $infile, '<:encoding(UTF-8)', $file;
 
     # save digested POD to the string $document
-    get_pod_parser()->parse_from_file($file, $handle);
+    get_pod_parser()->parse_from_filehandle($infile, $handle);
 
     my @words = _get_spellcheck_results($document);
 
@@ -124,6 +127,7 @@ sub pod_file_spelling_ok {
     # remove stopwords, select unique errors
     my $WL = \%Pod::Wordlist::Wordlist;
     @words = grep { !$WL->{$_} && !$WL->{lc $_} } @words;
+    $ALL_WORDS{$_}++ for @words;
     my %seen;
     @seen{@words} = ();
     @words = sort keys %seen;
@@ -140,7 +144,7 @@ sub pod_file_spelling_ok {
 
 sub all_pod_files_spelling_ok {
     my @files = all_pod_files(@_);
-
+    local %ALL_WORDS;
     if (!has_working_spellchecker()) {
         return $TEST->plan(skip_all => "no working spellchecker found");
     }
@@ -152,6 +156,19 @@ sub all_pod_files_spelling_ok {
         local $Test::Builder::Level = $Test::Builder::Level + 1;
         pod_file_spelling_ok($file) or undef $ok;
     }
+    if ( keys %ALL_WORDS ) {
+        # Invert k => v to v => [ k ]
+        my %values;
+        push @{ $values{ $ALL_WORDS{$_} } }, $_ for keys %ALL_WORDS;
+
+        my $labelformat = q[%6s: ];
+        my $indent      = q[ ] x 10;
+
+        $TEST->diag(qq[\nAll incorrect words, by number of occurrences:\n] .
+          join qq[\n], map { wrap( ( sprintf $labelformat, $_ ), $indent, join q[, ], sort @{ $values{$_} } ) }
+          sort { $a <=> $b } keys %values
+        );
+    }
     return $ok;
 }
 
@@ -305,7 +322,7 @@ L<add_stopwords> will be changed and documented properly.
 
 =head2 all_pod_files_spelling_ok( [@files/@directories] )
 
-Checks all the files for POD spelling. It gathers L<all_pod_files()> on each
+Checks all the files for POD spelling. It gathers L</all_pod_files> on each
 file/directory, and declares a L<Test::More/plan> for you (one test for each
 file), so you must not call C<plan> yourself.
 
@@ -373,6 +390,9 @@ can will use work on the current system. You can use this to skip tests if
 there is no spellchecker. Note that L</all_pod_files_spelling_ok> will do this
 for you.
 
+A full list of spellcheckers which this method might test can be found in the
+source of the C<spellchecker_candidates> method.
+
 =head2 set_spell_cmd($command)
 
 If you want to force this module to use a particular spellchecker, then you can
@@ -425,4 +445,3 @@ You may use, modify, and distribute this package under the
 same terms as Perl itself.
 
 =cut
-
